@@ -203,7 +203,37 @@ namespace Apache.Arrow.Tests
             await TestRoundTripRecordBatchAsync(originalBatch);
         }
 
-        private static void TestRoundTripRecordBatches(List<RecordBatch> originalBatches, IpcOptions options = null)
+        [Theory]
+        [InlineData(0, 45)]
+        [InlineData(3, 45)]
+        [InlineData(16, 45)]
+        public void WriteSlicedArrays(int sliceOffset, int sliceLength)
+        {
+            var originalBatch = TestData.CreateSampleRecordBatch(length: 100);
+            var slicedArrays = originalBatch.Arrays
+                .Select(array => ArrowArrayFactory.Slice(array, sliceOffset, sliceLength))
+                .ToList();
+            var slicedBatch = new RecordBatch(originalBatch.Schema, slicedArrays, sliceLength);
+
+            TestRoundTripRecordBatch(slicedBatch, strictCompare: false);
+        }
+
+        [Theory]
+        [InlineData(0, 45)]
+        [InlineData(3, 45)]
+        [InlineData(16, 45)]
+        public async Task WriteSlicedArraysAsync(int sliceOffset, int sliceLength)
+        {
+            var originalBatch = TestData.CreateSampleRecordBatch(length: 100);
+            var slicedArrays = originalBatch.Arrays
+                .Select(array => ArrowArrayFactory.Slice(array, sliceOffset, sliceLength))
+                .ToList();
+            var slicedBatch = new RecordBatch(originalBatch.Schema, slicedArrays, sliceLength);
+
+            await TestRoundTripRecordBatchAsync(slicedBatch, strictCompare: false);
+        }
+
+        private static void TestRoundTripRecordBatches(List<RecordBatch> originalBatches, IpcOptions options = null, bool strictCompare = true)
         {
             using (MemoryStream stream = new MemoryStream())
             {
@@ -223,13 +253,13 @@ namespace Apache.Arrow.Tests
                     foreach (RecordBatch originalBatch in originalBatches)
                     {
                         RecordBatch newBatch = reader.ReadNextRecordBatch();
-                        ArrowReaderVerifier.CompareBatches(originalBatch, newBatch);
+                        ArrowReaderVerifier.CompareBatches(originalBatch, newBatch, strictCompare: strictCompare);
                     }
                 }
             }
         }
 
-        private static async Task TestRoundTripRecordBatchesAsync(List<RecordBatch> originalBatches, IpcOptions options = null)
+        private static async Task TestRoundTripRecordBatchesAsync(List<RecordBatch> originalBatches, IpcOptions options = null, bool strictCompare = true)
         {
             using (MemoryStream stream = new MemoryStream())
             {
@@ -249,20 +279,20 @@ namespace Apache.Arrow.Tests
                     foreach (RecordBatch originalBatch in originalBatches)
                     {
                         RecordBatch newBatch = reader.ReadNextRecordBatch();
-                        ArrowReaderVerifier.CompareBatches(originalBatch, newBatch);
+                        ArrowReaderVerifier.CompareBatches(originalBatch, newBatch, strictCompare: strictCompare);
                     }
                 }
             }
         }
 
-        private static void TestRoundTripRecordBatch(RecordBatch originalBatch, IpcOptions options = null)
+        private static void TestRoundTripRecordBatch(RecordBatch originalBatch, IpcOptions options = null, bool strictCompare = true)
         {
-            TestRoundTripRecordBatches(new List<RecordBatch> { originalBatch }, options);
+            TestRoundTripRecordBatches(new List<RecordBatch> { originalBatch }, options, strictCompare: strictCompare);
         }
 
-        private static async Task TestRoundTripRecordBatchAsync(RecordBatch originalBatch, IpcOptions options = null)
+        private static async Task TestRoundTripRecordBatchAsync(RecordBatch originalBatch, IpcOptions options = null, bool strictCompare = true)
         {
-            await TestRoundTripRecordBatchesAsync(new List<RecordBatch> { originalBatch }, options);
+            await TestRoundTripRecordBatchesAsync(new List<RecordBatch> { originalBatch }, options, strictCompare: strictCompare);
         }
 
         [Fact]
@@ -541,6 +571,10 @@ namespace Apache.Arrow.Tests
         public void WriteMultipleDictionaryArrays()
         {
             List<RecordBatch> originalRecordBatches = CreateMultipleDictionaryArraysTestData();
+            Assert.Equal("RecordBatch: 10 columns by 3 rows", originalRecordBatches[0].ToString());
+            Assert.Equal("Schema: Num fields=10, Num metadata=0", originalRecordBatches[0].Schema.ToString());
+            Assert.Equal("Field: Name=dictionaryField_int8, DataType=dictionary, IsNullable=False, Metadata count=0",
+                originalRecordBatches[0].Schema.FieldsLookup["dictionaryField_int8"].Single().ToString());
             TestRoundTripRecordBatches(originalRecordBatches);
         }
 
